#include <msp430.h>
#include <stdint.h>

#include "../rf/RF_Connection.h"
#include "../rf/RF1A.h"
#include "../serial/serial.h"
#include "command.h"
//#include "LCD1x9.h"

#define BUFFER_SIZE	60
#define DELAY		1

//#define PWM_PERIOD	255

//Definitions
#define LED_On			P1OUT |= 0x01
#define LED_Off	        P1OUT &= (~0x01)
#define LED_Togg		P1OUT ^= 0x01

#define BIT14	0x4000;
#define BIT15	0x8000;

volatile unsigned char wor = 0;
volatile uint8_t rssi[(CHANNEL_STOP-CHANNEL_START)/CHANNEL_STEP][2] = {{0 , 0}};
volatile uint8_t loc = 0;
volatile uint8_t buffer[BUFFER_SIZE];
volatile uint8_t heard[NB_BANGLES];
volatile uint8_t h = 0;

void SerialWrite(unsigned char car){
	while (!(UCA0IFG&UCTXIFG));       // USCI_A0 TX buffer ready?
	UCA0TXBUF = car;            // TX -> RXed character
}

//Waitting for the program, we simulate a position
void position(int x, int y, int pos[2]){
	pos[0] = x;
	pos[1] = y;
}

//Send the temporary position to the station
void send_pos(int pos[2])
{
	buffer[0] = 3;
	buffer[1] = TRANSMITTER_ID;
	buffer[2] = 0x70;
	buffer[3] = pos[0];
	buffer[4] = pos[1];
	rf_transmit(buffer, 5);
}

//Send the different bangles it has heard
void send_heard(uint8_t heard[NB_BANGLES]){
	int b=3;	
	buffer[0] = 3;
	buffer[1] = TRANSMITTER_ID;
	buffer[2] = 0x69;
	for (b=0 ; b = h; b++)
		buffer[b+3]=heard[b];
	rf_transmit(buffer, h+3);
}

void packet_received(void)
{
	//LED_Togg;
	//LED_On;
  	//LCD1x9_Write("ok");
	uint8_t RxBufferLength;
	uint8_t RxBuffer[BUFFER_SIZE];
	int i;
	int ind;

	RxBufferLength = ReadSingleReg(RXBYTES);
	ReadBurstReg(RF_RXFIFORD, RxBuffer, RxBufferLength);

	while (!(UCA0IFG&UCTXIFG));       // USCI_A0 TX buffer ready?
	UCA0TXBUF = RxBufferLength;            // TX -> RXed character
	for(i=0; i<RxBufferLength; i++)
	{
		while (!(UCA0IFG&UCTXIFG));       // USCI_A0 TX buffer ready?
		UCA0TXBUF = RxBuffer[i];            // TX -> RXed character
	}


	switch(RxBuffer[2])
	{

               //For frequency change
		case 'f':
			ChangeChannel(RxBuffer[3]);
			
			//Save the channel and the RSSI in rssi
			ind = (RxBuffer[3]-CHANNEL_START)/CHANNEL_STEP;
			rssi[ind][0] = RxBuffer[3];
			rssi[ind][1] = RxBuffer[5];
			//SerialWrite(0x02);
			//SerialWrite(rssi[ind][0]);	
			//SerialWrite(rssi[ind][1]);	
			if (( RxBuffer[3] + CHANNEL_STEP ) > CHANNEL_STOP){
				ReceiveOff();				
				loc = 0x01;
				break;
			}	
			else
				ReceiveOn();
			while((Strobe(RF_SNOP) & 0x70) != 0x10);
			break;
		case 'p':
			heard[h] = RxBuffer[1];
			h++;
		default : 
			break;
	}
}


void init_led(void)
{
	P1OUT &= ~BIT0;
	P1DIR |= BIT0;
//	P3OUT = 0x00;
//	P3DIR |= 0x01;
}

void port_mapping(void)
{
  PMAPPWD = 0x02D52;            // Get write-access to port mapping regs
  P3MAP0 = PM_TA1CCR1A;       	// Map TA1CCR1 output to P3.0
  P3MAP1 = PM_TA1CCR2A;         // Map TA1CCR2 output to P3.1
  P3MAP2 = PM_TA0CCR1A;         // Map TA0CCR1 output to P3.2
  P3MAP3 = PM_TA0CCR2A;         // Map TA0CCR2 output to P3.3
  P3MAP4 = PM_TA0CCR3A;         // Map TA0CCR3 output to P3.4
  P3MAP5 = PM_TA0CCR4A;         // Map TA0CCR4 output to P3.5
  PMAPPWD = 0;                  // Lock port mapping registers
  P3DIR |= 0x3F;          		// P3.0 to P3.5 as output
  P3SEL |= 0x3F;     		    // P3.0 and P3.3 options select
}


int main(void)
{
  	WDTCTL = WDTPW + WDTHOLD;                 // Stop WDT
	
	//For debug
	int i;
	int pos[2];
	init_led();
	uart_init();
	//while(1){
	for(i=0; i<5; i++){
		LED_On;
		__delay_cycles(100000);
		LED_Off;
		__delay_cycles(100000);
	}

	rf_init();



	// enable interrupts
	_BIS_SR(GIE);


	ChangeChannel(CHANNEL_START);
	ReceiveOn();
  	//Wait for RX status to be reached
	while((Strobe(RF_SNOP) & 0x70) != 0x10);
	
//  	LCD1x9_Write("demo rssi");
	
	//SerialWrite('C'); SerialWrite('0');SerialWrite('\n');


	//Init to channel CHANNEL_START
	//ChangeChannel(CHANNEL_START);

	while(1){
		while(!loc)
		{
			//SerialWrite(0x00);
			//__delay_cycles(100000);
		}
		loc = 0;
		ChangeChannel(0);
		position(3, 5, pos); //Would be the algorithm
		ReceiveOn();
		SerialWrite(0x01);
		SerialWrite(0x02);
		owndelay(DELAY*TRANSMITTER_ID); 
		send_pos(pos);
		owndelay(DELAY*NB_BANGLES);
		//SerialWrite(0x01);
		//SerialWrite(0x02);
		//send_heard(heard);
		ReceiveOff();
		//SerialWrite(0x01);
		//SerialWrite(0x02);		
	}
	return 0;
}
