#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <sys/stat.h>
//#include <sys/types.h>
#include <sys/mman.h>

#define BACKGROUND_FILE	"data/background"
#define FILENAME_SIZE	128
#define MAX_IMAGE		255
#define DEST_REP		"generated"


struct image_file_s 
{
	char filename[FILENAME_SIZE];
	char *addr;
	int fd;
};

typedef struct image_file_s image_file_t;


struct images_s
{
	int id;
	char print;
	int nb_image;
	char image[MAX_IMAGE][3];
};

typedef struct images_s images_t;

void generate_header(image_file_t *images, int size)
{
	int i;

	FILE *header = fopen("files_index.h", "w");
	fprintf(header, "#ifndef __FILES_INDEX_H__\n");
	fprintf(header, "#define __FILES_INDEX_H__\n\n");
	for(i=0; i<size; i++)
	{
		fprintf(header, "#define %s %d\n",images[i].filename, i);
	}
	fprintf(header, "\n#endif\n");
	fclose(header);

}

void generate_files(images_t *images, int size)
{
	int i,j;
	char pathname[FILENAME_SIZE];
//	struct stat stat_dir;

	// test if the destination directory exist
//	if(stat(DEST_REP, &stat_dir)==-1)
//		mkdir(DEST_REP, S_IRWXU);


	for(i=0; i<size; i++)
	{
		if(images[i].print != 'X')
		{
			sprintf(pathname, "%s/image_%i.h", DEST_REP, images[i].id);
			FILE *file = fopen(pathname, "w");
			fprintf(file, "#ifndef __IMAGE_H__\n");
			fprintf(file, "#define __IMAGE_H__\n\n");
			fprintf(file, "#define BANGLE_ID	%d\n\n",i);
			fprintf(file, "uint8_t images[%d][3]={\n",images[i].nb_image);
			for(j=0; j<images[i].nb_image; j++)
			{
				fprintf(file, "{0x%hhx, 0x%hhx, 0x%hhx},\n",images[i].image[j][0], images[i].image[j][1], images[i].image[j][2]);
			}
			fprintf(file, "};\n");
			fprintf(file, "#endif\n");
		}
	}
}



int main(int argc, char *argv[])
{
	char *addr;
	struct stat sb, sb_background;
	//size_t length;
	char file[] = BACKGROUND_FILE;
	image_file_t *image_array;
	images_t *images;
	int nb_image = 0;
	int i,j;
	char buffer[7];
	unsigned long int temp;
	uint8_t red, green, blue;

	buffer[6] = '\0';

	if(argc > 1)
	{
		image_array = malloc(sizeof(image_file_t) * (argc-1));
	}

	// Open the background file
	int fd = open(file, O_RDONLY);

	// Get information about the file
	fstat(fd, &sb_background);

	addr = mmap(NULL, sb_background.st_size, PROT_READ, MAP_PRIVATE, fd, 0);
	images = malloc(sizeof(images_t) * sb_background.st_size);

	//for(i=0; i<sb.st_size; i++)
	//	printf("%c", addr[i]);

	// Open and map each file
	for(i=1; i<argc; i++)
	{
		strcpy(image_array[nb_image].filename, argv[i]);
		image_array[nb_image].fd = open(argv[i], O_RDONLY);
		fstat(image_array[nb_image].fd, &sb);
		image_array[nb_image].addr = mmap(NULL, sb.st_size, PROT_READ, MAP_PRIVATE, image_array[nb_image].fd, 0);
		nb_image++;
	}

	for(i=0; i<sb_background.st_size; i++)
	{
		images[i].id = i;
		images[i].nb_image = argc-1;

		if(addr[i] != 'X')
		{
			images[i].print = '_';
			for(j=0; j<argc-1; j++)
			{
				memcpy(buffer, image_array[j].addr, 6);
				temp = strtol(buffer, NULL, 16);
				if(temp == 0xFFFFFF)
				{
					blue = 0;
					green = 0;
					red = 0;
				}
				else
				{
					blue = temp & 0x00FF;
					green = temp>>8 ;
					red = temp>>16 ;
				}
				printf("value t:%lx:red:%x:green:%x:blue:%x:\n", temp, red, green, blue);
				images[i].image[j][0] = red;
				images[i].image[j][1] = blue;
				images[i].image[j][2] = green;
				image_array[j].addr += 6;
			}
		}
		else
		{
			images[i].print = 'X';
			for(j=0; j<argc-1; j++)
			{
				image_array[j].addr += 6;
			}
		}
	}

	generate_files(images, sb_background.st_size);
	generate_header(image_array, argc-1);

	return 0;
}

